import React, { useState, useRef, useCallback, useEffect } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var processCloudcastKeyUrl = function (cloudcastKey) {
    if (!cloudcastKey)
        return { modifiedKey: "", orderValue: "" };
    var modifiedKey = cloudcastKey;
    if (!cloudcastKey.includes("mixcloud.com")) {
        modifiedKey = "https://www.mixcloud.com" + cloudcastKey;
    }
    var urlObject = new URL(modifiedKey);
    var orderValue = urlObject.searchParams.get("order") || "popular";
    urlObject.searchParams.delete("order");
    modifiedKey = urlObject.toString().replace("https://www.mixcloud.com", "");
    return { modifiedKey: modifiedKey, orderValue: orderValue };
};
var fetchShows = function (_a) {
    var cloudcastKey = _a.cloudcastKey, _b = _a.limit, limit = _b === void 0 ? 100 : _b, _c = _a.withExclusives, withExclusives = _c === void 0 ? false : _c;
    return __awaiter(void 0, void 0, void 0, function () {
        var _d, modifiedKey, orderValue, getFilteredShows, response, data, cloudcastsEndpoint, cloudcastsResponse, _e, cloudcastsData, _f;
        var _g, _h, _j;
        return __generator(this, function (_k) {
            switch (_k.label) {
                case 0:
                    if (!cloudcastKey)
                        return [2 /*return*/, []];
                    _d = processCloudcastKeyUrl(cloudcastKey), modifiedKey = _d.modifiedKey, orderValue = _d.orderValue;
                    getFilteredShows = function (data) {
                        return data === null || data === void 0 ? void 0 : data.filter(function (show) { return (withExclusives ? show : !show.is_exclusive); });
                    };
                    if (!modifiedKey) return [3 /*break*/, 9];
                    return [4 /*yield*/, fetch("https://api.mixcloud.com".concat(modifiedKey, "?metadata=1&limit=").concat(limit))];
                case 1:
                    response = _k.sent();
                    return [4 /*yield*/, response.json()];
                case 2:
                    data = _k.sent();
                    if (!(data.type && data.type === "cloudcast")) return [3 /*break*/, 3];
                    return [2 /*return*/, getFilteredShows([data])];
                case 3:
                    if (!(data.data && data.data.length > 0)) return [3 /*break*/, 4];
                    return [2 /*return*/, getFilteredShows(data.data)];
                case 4:
                    if (!((_g = data.metadata) === null || _g === void 0 ? void 0 : _g.connections)) return [3 /*break*/, 9];
                    cloudcastsEndpoint = ((_h = data.metadata.connections) === null || _h === void 0 ? void 0 : _h.cloudcasts) ||
                        ((_j = data.metadata.connections) === null || _j === void 0 ? void 0 : _j[orderValue]);
                    _e = cloudcastsEndpoint;
                    if (!_e) return [3 /*break*/, 6];
                    return [4 /*yield*/, fetch(cloudcastsEndpoint + "?metadata=1&limit=".concat(limit))];
                case 5:
                    _e = (_k.sent());
                    _k.label = 6;
                case 6:
                    cloudcastsResponse = _e;
                    _f = cloudcastsResponse;
                    if (!_f) return [3 /*break*/, 8];
                    return [4 /*yield*/, cloudcastsResponse.json()];
                case 7:
                    _f = (_k.sent());
                    _k.label = 8;
                case 8:
                    cloudcastsData = _f;
                    if (cloudcastsData.data && cloudcastsData.data.length > 0) {
                        return [2 /*return*/, getFilteredShows(cloudcastsData.data)];
                    }
                    _k.label = 9;
                case 9: return [2 /*return*/, []];
            }
        });
    });
};

var MixcloudPlayer = function (_a) {
    var _b, _c;
    var cloudcastKey = _a.cloudcastKey, _d = _a.autoPlay, autoPlay = _d === void 0 ? true : _d, _e = _a.initShowIndex, initShowIndex = _e === void 0 ? 0 : _e, _f = _a.withExclusives, withExclusives = _f === void 0 ? false : _f, onReady = _a.onReady;
    var _g = useState(false), scriptLoaded = _g[0], setScriptLoaded = _g[1];
    var _h = useState(null), isPlaying = _h[0], setIsPlaying = _h[1];
    var _j = useState(null); _j[0]; var setDuration = _j[1];
    var _k = useState(null), widget = _k[0], setWidget = _k[1];
    var _l = useState([]), shows = _l[0], setShows = _l[1];
    var _m = useState(null), showIndex = _m[0], setShowIndex = _m[1];
    var _o = useState(null), loading = _o[0], setLoading = _o[1];
    var iframeRef = useRef(null);
    var timer = useRef(null);
    var incrementShowIndex = useCallback(function () {
        showIndex !== null &&
            showIndex < shows.length - 1 &&
            setShowIndex(showIndex + 1);
    }, [showIndex, shows]);
    useEffect(function () {
        var script = document.createElement("script");
        script.src = "https://widget.mixcloud.com/media/js/widgetApi.js";
        script.async = true;
        document.body.appendChild(script);
        script.onload = function () { return setScriptLoaded(true); };
    }, []);
    useEffect(function () {
        if (!cloudcastKey)
            return;
        console.log("loading shows...");
        setLoading(true);
        fetchShows({ cloudcastKey: cloudcastKey, withExclusives: withExclusives })
            .then(function (shows) {
            setShows(shows);
            setShowIndex((shows === null || shows === void 0 ? void 0 : shows.length) > initShowIndex ? initShowIndex : 0);
            console.log("loaded shows ", shows);
        })
            .finally(function () { return setLoading(false); });
    }, [cloudcastKey, withExclusives, initShowIndex]);
    useEffect(function () {
        if (!shows || shows.length === 0)
            return;
        if (iframeRef.current && scriptLoaded) {
            var widget_1 = window.Mixcloud.PlayerWidget(iframeRef.current);
            setWidget(null); // so that we can clear controls until new widget ready
            setLoading(true);
            widget_1.ready.then(function (player) {
                console.log("widget ready");
                setWidget(widget_1);
                //widget.pause();
                onReady === null || onReady === void 0 ? void 0 : onReady(widget_1);
                widget_1.events.pause.on(function () {
                    console.log("Widget pause");
                    setIsPlaying(false);
                    setLoading(false);
                });
                widget_1.events.play.on(function () {
                    console.log("Widget play");
                    setIsPlaying(true);
                    setLoading(false);
                });
                widget_1.events.ended.on(function () { return console.log("Widget ended"); });
                widget_1.events.progress.on(function () { return console.log("Widget progress"); });
                widget_1.events.buffering.on(function () {
                    console.log("Widget buffering");
                    setLoading(true);
                });
                widget_1.events.error.on(function () {
                    console.log("Widget error");
                    incrementShowIndex();
                });
                // error hit when accessing exclusive content
                widget_1.getDuration().then(function (duration) {
                    console.log("getDuration from ready", duration);
                    setLoading(false);
                    if (!duration) {
                        console.error("duration not found so probably licence issue");
                        incrementShowIndex();
                        return;
                    }
                    setDuration(duration);
                    autoPlay === true && widget_1.play();
                });
            });
            return function () {
                console.log("return");
                timer.current && clearTimeout(timer.current);
            };
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [shows, showIndex, scriptLoaded]);
    useEffect(function () {
        console.log(loading ? "loading" : "loaded");
    }, [loading]);
    var handleToggle = function () {
        if (!widget)
            return;
        widget.togglePlay();
    };
    var handlePrev = function () {
        if (!widget || showIndex === 0)
            return;
        // widget.pause();
        timer.current = setTimeout(function () { return showIndex !== null && setShowIndex(showIndex - 1); }, 200);
    };
    var handleNext = function () {
        if (!widget || showIndex === null || showIndex >= shows.length - 1)
            return;
        // widget.pause();
        timer.current = setTimeout(function () { return incrementShowIndex(); }, 200);
    };
    return (React.createElement("div", { "data-testid": "mixcloud-player" },
        shows.length > 0 && (React.createElement(React.Fragment, null,
            React.createElement("iframe", { title: "mixcloud-widget", ref: iframeRef, key: (_b = shows[showIndex]) === null || _b === void 0 ? void 0 : _b.key, width: "100%", height: "60", allow: "autoplay", style: { visibility: "visible" }, src: "https://player-widget.mixcloud.com/widget/iframe/?hide_cover=1&mini=1&autoplay=".concat(autoPlay, "&feed=").concat(encodeURIComponent((_c = shows[showIndex]) === null || _c === void 0 ? void 0 : _c.url)), frameBorder: "0" }),
            widget && (React.createElement(React.Fragment, null,
                React.createElement("div", { style: { marginTop: "10px" } },
                    React.createElement("button", { onClick: handleToggle }, isPlaying ? "Pause" : "Play")),
                React.createElement("div", { style: { marginTop: "10px" } },
                    React.createElement("button", { onClick: handlePrev }, "Prev")),
                React.createElement("div", { style: { marginTop: "10px" } },
                    React.createElement("button", { onClick: handleNext }, "Next")))))),
        loading && React.createElement("p", { "data-testid": "mixcloud-player__loading" }, "loading!!")));
};

export { MixcloudPlayer as default };
